# Copyright 2005-2008, Google, Inc.

# This software is provided as an example of using the Ruby interface
# to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Original Author: Scott Lininger
# Special thanks to Todd Burch who took my junky demo code and make it fit
# for public consumption. Credit for the castle level, as well as the
# beautifully re-drawn Prince IO goes to Tyson Kartchner!
#-----------------------------------------------------------------------------

# This runs the Ruby side of the Prince IO game.

class KnightGame

@@camera_softness = 15.0 ; 
@@vector_down = Geom::Vector3d.new 0, 0, -1  # aim downwards 
@@vector_left = Geom::Vector3d.new -1, 0, 0
@@vector_right = Geom::Vector3d.new 1, 0, 0
@@path   = "Plugins/knightgame"
@@file   = "knightgame.html" ; 
@@bullet = "laser.wav" ; 
@@sword  = "Free-Loops.com-1180912398.wav" ; 
@@fall   = "scream.mp3" ;
@@knight_dlg = nil ;

def initialize()

	
	@entities = {}
	@falling = 0 ; 
	@knight_file = Sketchup.find_support_file(@@file, @@path ) ; 
	if (!@knight_file) then 
		UI.messagebox("Cannot find file '#{@@file} in folder '#{@@path}'") ; 
		return ;
	end ; 

	@bullet_sound = Sketchup.find_support_file(@@bullet, @@path ) ; 
	if (!@bullet_sound) then 
		UI.messagebox("Cannot find file '#{@@bullet} in folder '#{@@path}'") ; 
		return ;
	end ; 

	@sword_sound = Sketchup.find_support_file(@@sword, @@path ) ; 
	if (!@sword_sound) then 
		UI.messagebox("Cannot find file '#{@@sword} in folder '#{@@path}'") ; 
		return ;
	end ; 
	
	@falling_sound = Sketchup.find_support_file(@@fall, @@path ) ; 
	if (!@falling_sound) then 
		UI.messagebox("Cannot find file '#{@@fall} in folder '#{@@path}'") ; 
		return ;
	end ; 
	
  if @@knight_dlg == nil
    @@knight_dlg = UI::WebDialog.new("Knight Dialog", false, "Knight Dialog", 1024, 200, 100, 0, true);

    @@knight_dlg.add_action_callback("push_frame") do |d,p| 
      push_frame(d,p) 
    end
  end
	
	@@knight_dlg.set_file(@knight_file)
	@@knight_dlg.show() ; 
end ; 

#
# push_frame is called 15 times a second (see variable "delay") from Javascript
# 
# In push_frame, there several things to do: 
# 
# 1) Analyze the state of the game.  The state is changed by keypresses.  
#    State information is passed from Javascript through variable "params" 
#
# 2) Adjust the visibility of the Hero component to show the action the 
#    the user requested. 
#
# 3) Run ray traces to find any obstacles and the ground plane Hero will stand on
#
# 4) Tweak the camera settings to adjust for jumping, running, shooting, etc. 
#

def push_frame(dialog,data)

	params = query_to_hash(data)   # parse state information 
	
	entity = find_entity_by_name(params['name'])
	entity.transform! entity.transformation.inverse

	# See if Hero needs to turn directions. 
	
	target_rotz = params['rotz'].to_f  
	radians = target_rotz ;
	new_rotz = Geom::Transformation.rotation entity.transformation.origin, entity.transformation.zaxis, radians
	entity.transform! new_rotz

	
	# variables x, y and z represent ... 
	
	vector = Geom::Vector3d.new params['x'].to_f, params['y'].to_f, params['z'].to_f
	translation = Geom::Transformation.translation vector
	entity.transform! translation

	#
	# Run a Ray Trace to find the surface Hero will walk / land on 
	#
	
	position = Geom::Point3d.new params['x'].to_f, params['y'].to_f, params['z'].to_f 
	
	hit_path = Sketchup.active_model.raytest [position,@@vector_down] 
	
	if hit_path != nil
		groundZ = hit_path[0].z.to_f   # "ground" is the first surface hit. 
		@falling = 0 ; 
	else
		groundZ = -5000.0 ;    # else fall off into la-la land 
		@falling += 1; 
		UI.play_sound @falling_sound if @falling == 4 ; 

	end
	script = "top.GROUNDZ = " + groundZ.to_s + ";"
	
	#
	# There's a bug in execute_script that puts garbage on the end of the value.
	# To work around it, leave blank before the closing paren, and then clean 
	# it up in javascript. 
	#
	if RUBY_PLATFORM.include?("mswin") then script = "top.GROUNDZ = " + groundZ.to_s + ";"
	else script = "newGroundZ(" + groundZ.to_s + " );" ;  
	end ; 
	dialog.execute_script(script);

	#
	# Run a ray trace to look for a left obstacle.  If Hero is a certain distance 
	# from the obstacle, and he is facing it and he swipes his sword at it, 
	# the obstacle will be erased. 
	# 
	
	hit_path = Sketchup.active_model.raytest [position, @@vector_left] ; 
	if hit_path != nil
		obs_left = params['x'].to_f - hit_path[0].x.to_f
		UI.play_sound @sword_sound if params['sword'].to_i > 0 ; 
		if obs_left < 100 && target_rotz < -1.0 && params['sword'].to_i > 0
			hit_path[1][0].erase!
			obs_left = 5000.0  # no more obstacles 
		end
	else
		obs_left = 5000.0  # no obstacles 
	end
	script = "top.OBSTACLE_LEFT = " + obs_left.to_s + ";"
	dialog.execute_script(script) ;

	#
	# Run the same ray trace to the right
	# 

	hit_path = Sketchup.active_model.raytest [position, @@vector_right] 
	if hit_path != nil
		obs_right = hit_path[0].x.to_f - params['x'].to_f
		UI.play_sound @sword_sound if params['sword'].to_i > 0 ; 
		if obs_right < 100 && target_rotz > -1.0 && params['sword'].to_i > 0
			hit_path[1][0].erase!
			obs_right = 5000.0
		end
	else
		obs_right = 5000.0
	end
	script = "top.OBSTACLE_RIGHT = " + obs_right.to_s + ";"
	dialog.execute_script(script);

	#
	# Change the visibility of the Hero components based on the current frame.
	# 
	
	for subentity in entity.definition.entities
		if subentity.definition.name == params['frame']
			subentity.hidden = false
		else
			subentity.hidden = true
		end
	end

	
	# Adjust the camera settings  
	
	view = Sketchup.active_model.active_view
	old_x = view.camera.eye.x
	old_y = view.camera.eye.y
	old_z = view.camera.eye.z

	new_x = params['x'].to_f
	new_y = params['y'].to_f - 500.0
	new_z = params['z'].to_f + 200.0

	x = (new_x + old_x * @@camera_softness) / ( @@camera_softness + 1.0);
	y = (new_y + old_y * @@camera_softness) / ( @@camera_softness + 1.0);
	z = (new_z + old_z * @@camera_softness) / ( @@camera_softness + 1.0);

	newEye = Geom::Point3d.new(x,y,z);

	@targetX = 0.0 if !@targetX ; 
	@targetY = 0.0 if !@targetY ; 
	@targetZ = 0.0 if !@targetZ ; 

	@targetY = (params['y'].to_f + @targetY)/2.0 ; 
	@targetX = (params['x'].to_f + @targetX)/2.0 ; 

	# If "#1" is appended to the frame name, Hero was just shooting.  
	# Therefore, tweak the camera target to show the force of the gun
	
	if params['frame'].index('#1') != nil
		if @just_shot == false
			if target_rotz < -1.0
				@targetX -= 5
			else
				@targetX += 5
			end
		end
		UI.play_sound @bullet_sound if !@just_shot; 
		@just_shot = true
	else
		@just_shot = false
	end
	
	
	if params['z'].to_f > 0.0 || groundZ > -4000.0
		if groundZ < 0.0
			visualZ = 0.0
		else
			visualZ = groundZ
		end
		@targetZ = (@targetZ * 5.0 + (params['z'].to_f + visualZ) / 2.0 + 50.0 ) / 6.0
	else
		@targetZ = (@targetZ*80.0 + (params['z'].to_f+groundZ)/2.0+50.0 )/81.0
	end
  
	newTarget = Geom::Point3d.new(@targetX,@targetY,@targetZ) 
	newUp = Geom::Vector3d.new(0,0,1)

	newCamera = view.camera
	newCamera.set(newEye,newTarget,newUp)
	view.camera = newCamera
	view.invalidate()
end

def find_entity_by_name(name)
	if @entities[name] != nil
		return @entities[name]
	else
		for entity in Sketchup.active_model.entities
      if entity.typename == "ComponentInstance"
        if entity.name.to_s == name
          return entity
			  end
      end
		end
	end
end

def unescape(string)
	if string != nil
		string = string.gsub(/\+/, ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
			[$1.delete('%')].pack('H*')
		end
	end
	return string
end

def query_to_hash(query)
	param_pairs = query.split('&')
	param_hash = {}
	for param in param_pairs
		name, value = param.split('=')
		name = unescape(name)
		value = unescape(value)
		param_hash[name] = value
	end
	return param_hash
end

end # class KnightGame 

if not file_loaded?("knightgame.rb")
  knight_menu = UI.menu('Plugins').add_submenu('Knight Game')

  knight_menu.add_item('Map 1: Simple Example') { 
    file = Sketchup.find_support_file("Map_Simple.skp" , "Plugins/knightgame") ; 
    Sketchup.open_file file ; 
    KnightGame.new() ; 
  }
  knight_menu.add_item('Map 2: The Castle Gate') { 
    file = Sketchup.find_support_file("Map_Castle.skp" , "Plugins/knightgame") ; 
    Sketchup.open_file file ; 
    KnightGame.new() ; 
  }
  knight_menu.add_item('Map 3: The Spooky Forest') { 
    file = Sketchup.find_support_file("Map_Forest.skp" , "Plugins/knightgame") ; 
    Sketchup.open_file file ; 
    KnightGame.new() ; 
  }
end

file_loaded("knightgame.rb")   # Mark the script loaded. 
